pub trait SpiMasterDmaChannel<'d, S>: Sealed + Into<AhbGdmaChannel<'d>> { }Available on crate feature
unstable only.Expand description
DMA channel trait for SPI peripherals.
Implemented for each channel type that can serve a particular SPI instance S.
Stability
This API is marked as unstable and is only available when the unstable
crate feature is enabled. This comes with no stability guarantees, and could be changed
or removed at any time.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.